function xepm_mapping() {
	this.url = "./";
	this.models = [];
	this.templates = [];
}

xepm_mapping.prototype.scan_devices = function() {
	var filter_results = !!$("input#filter_scan").prop("checked");
	var subnet_field = $("input#xepm_subnet");
	var subnet = subnet_field.val();
	if(subnet.length !== 0) {
		var scan_button = $("td.scan_subnet_button");
		scan_button.attr('disabled','disabled');

		xepm_mapping.clear_unassinged_devices();
		$('table.unassigned_table').css({'display':'none'}); //Hide unassigned devices table
		$('i#scan_subnet_spinner').css({'display':'block'}); //Show spinner

		$.ajax({
			type: "POST",
			url: xepm_mapping.url,
			dataType: 'json',
			data: {
				class: 'endpoint',
				method: 'scan_subnet',
				scan_subnet: subnet
			},
			success: function (data) {
				xepm_mapping.scanned_devices = data;
				var table = $('div.epm_unassigned_devices > table').find('tbody');
				$.each(data, function(index, device) {
					if (!filter_results || (this.brand_id > 0)) {
						var brands = $('select#epm_device_mapping_brands').clone(); // Clone brands dropdown field
						brands.attr({
							'id': null,
							'name' : 'unmapped_brand_id[]'
						});
						xepm_mapping.add_device(this.display_mac_address, this.ip_address, this.brand_id);
					}
				});
				$('table.unassigned_table').css({'display':'block'}); //Unhide unassigned devices table
				scan_button.removeAttr('disabled');
			},
			complete: function (data) {
				$('i#scan_subnet_spinner').css({'display':'none'});
			}
		});
	}
};

// Remove all unassigned devices
xepm_mapping.prototype.clear_unassinged_devices = function() {
	$('div.epm_unassigned_devices > table > tbody').find('tr').remove();
};

// Opens new browsers tab with device cfg.
xepm_mapping.prototype.view_cfg = function(mac_address) {
	var url = "/xepm-provision/" + mac_address + ".cfg";
	window.open(url, '_blank');
};

// Move assigned device to unassigned on delete device click
xepm_mapping.prototype.remove_device = function(element) {
	var row = $(element).parent().parent();
	var mac_address = row.children().eq(1).text();
	row.remove();
	xepm_mapping.add_device(mac_address, '');
};

// Create a new unmapped device
xepm_mapping.prototype.add_device = function(mac_address, ip_address, brand_id) {
	var models = $("<select/>", { name : "unmapped_model_id[]" }).append($("<option/>", { selected : true }));
	var templates = $("<select/>", { name : "unmapped_template_id[]" }).append($("<option/>", { selected : true }));
	var brands = $('select#epm_device_mapping_brands').clone(); // Clone brands dropdown field
	brands.attr({
		'id': null,
		'name' : 'unmapped_brand_id[]'
	});

	var row = $("<tr/>")
		.append($("<td/>").append($("<input/>", { type : "text", value : mac_address, name: "unmapped_mac_address[]" })))
		.append($("<td/>", { html: (ip_address === null) ? "-" : ip_address }))
		.append($("<td/>").append(brands))
		.append($("<td/>").append(models))
		.append($("<td/>").append(templates))
		.append($("<td/>")); // extensions

	if (brand_id > 0) {
		brands.val(brand_id);
		xepm_mapping.populate_on_brand_selection(row, brand_id);
	}

	brands.bind("change keyup", function(event) { xepm_mapping.handle_dropdown(event); });
	models.bind("change keyup", function(event) { xepm_mapping.handle_dropdown(event); });

	row.appendTo($("div.epm_unassigned_devices > table").find('tbody'));
	$('table.unassigned_table').css({'display':'block'}); //Unhide unassigned devices table
};

// Populate model dropdown on brand selection
xepm_mapping.prototype.populate_on_brand_selection = function(row, brand_id) {
	if (xepm_mapping.models[brand_id] === undefined) {
		var element = row.find("select[name*='model_id']");
		$.ajax({
			type: 'POST',
			url: xepm_mapping.url,
			dataType: 'json',
			data: {
				class: 'endpoint',
				method: 'models_by_brand',
				brand_id: brand_id
			},
			success: function (data) {
				element.empty();
				element.fill(data.models, function(model) {
					this.val(model.model_id);
					this.text(model.name);
				}, false);
				row.find("select[name*='template_id']").reset();
				row.find("select[name*='extensions']").remove();
				element.data("options_loaded", true);
			}
		});
	}
};

// Polulate template and extension dropdowns on model selection
xepm_mapping.prototype.populate_on_model_selection = function(row, model_id) {
	if (xepm_mapping.templates[model_id] === undefined) {
		var element = row.find("select[name*='extensions']");
		$.ajax({
			type: 'POST',
			url: xepm_mapping.url,
			dataType: 'json',
			data: {
				class: 'endpoint',
				method: 'templates_and_ext_by_model',
				model_id: model_id
			},
			success: function (data) {
				row.find("select[name*='template_id']").fill(data.templates, function(template) {
					this.val(template.template_id);
					this.text(template.name);
				}, false);
				var row_index = parseInt(row[0].rowIndex) - 1;
				var prefix = row.closest("table")[0].id.contains("unassigned") ? "unmapped" : "mapped";
				var cell = row.children().eq(5);
				if(prefix == "mapped") {
					if(element.length) {
						row_index = element.attr('name').match(/\[(\d+){1}\]/).pop(); // Get device index(row) to preserve extensions indexes
					}
				}
				element.remove();
				for (var i = 0; i < data.extension_count; i++) {
					var dropdown = $('<select/>', { name: prefix + "_extensions[" + row_index + "][" + i + "]"});
					dropdown.fill(data.extensions, function(extension) {
						this.val(extension.value);
						this.text(extension.content);
					}, false);
					dropdown.appendTo(cell);
				}
				element.data("options_loaded", true);
			}
		});
	}
};

//NOTE:
// If there any change, warn.
xepm_mapping.prototype.configure_button_listener = function(device_id) {
	if(!xepm_mapping.is_changed) {
		//Do something
	} else if(confirm("Are you sure you want to nevigate away? All unsaved changes will be lost")) {
		//Do something else
	}
};

// assign events to page elements
xepm_mapping.prototype.assign_events = function() {
	$("select").bind("change keyup", function(event) {
		xepm_mapping.is_changed = true;
		xepm_mapping.handle_dropdown(event);
	});
	$("input").bind("change keyup", function(event) {
		if (event.target.id != "xepm_subnet") {
			xepm_mapping.is_changed = true;
		}
	});
	$("input[name*='filter_scan']").bind("change", function(event) {
		var filter_results = !!$(event.target).prop("checked");
		if (xepm_mapping.scanned_devices !== undefined) {
			$("div.epm_unassigned_devices > table").find("tbody").empty();
			$.each(xepm_mapping.scanned_devices, function() {
				if (!filter_results || (this.brand_id > 0)) {
					xepm_mapping.add_device(this.display_mac_address, this.ip_address, this.brand_id);
				}
			});
		}
	});
};

// Process dropdown events
xepm_mapping.prototype.handle_dropdown = function(event) {
	var element = event.target;
	if (element.previous_value != element.value) {
		element.previous_value = element.value;
		var row = $(element).closest("tr");
		if (element.name.contains("brand_id")) {
			xepm_mapping.populate_on_brand_selection(row, element.value);
		} else if (element.name.contains("model_id")) {
			xepm_mapping.populate_on_model_selection(row, element.value);
		}
	}
};

// Returns true when a string contains the specified substring
if (!("contains" in String.prototype)) {
	String.prototype.contains = function(needle) {
		return this.indexOf(needle) != -1;
	}
}

// Removes the module
xepm_mapping.prototype.remove_module = function(event) {
	event.stopPropagation();
	var element = event.target;
	if(confirm("Are you sure you want to remove this module ?")) {
		var module_index = $(element).closest("li").index();
		$("ul#mapping_attached_modules li").eq(module_index).remove();
		$("div#mapping_modules div").eq(module_index).remove();
		xepm_mapping.attached_modules = null;
		if(typeof xepm_mapping.attached_modules != "undefined") {
			xepm_mapping.attached_modules = parseInt($("ul#mapping_attached_modules li").length) - 1;
		}
		if (xepm_mapping.visible_module == module_index) { xepm_mapping.show_last_module(); }
		$('button#xepm_add_module').prop('disabled', false);
	}
};

// Add a module to device
xepm_mapping.prototype.add_module = function(e) {
	xepm_mapping.attached_modules = null;
	if(typeof xepm_mapping.attached_modules != "undefined") {
		xepm_mapping.attached_modules = parseInt($("ul#mapping_attached_modules li").length);
	}
	var max_modules = $('#epm_device_conf_exp_modules #mapping_max_modules').val();
	var device_id = $('#modal-epm-device_configure #data').val();
	if(max_modules !== 'undefined' && !isNaN(parseInt(max_modules)) &&
		parseInt(max_modules) > 0 && device_id !== 'undefined' &&
		!isNaN(parseInt(device_id) && parseInt(device_id) > 0)
	) {
		if (xepm_mapping.attached_modules < parseInt(max_modules)) {
			var modules = $("select#mapping_module_id");
			var module_id = modules.val();
			if(parseInt(module_id) > 0) {
				$('button#xepm_add_module').prop('disabled', true);
				var module_name = modules.children(':selected').text();
				var attached_modules = $('input[name="mapping_attached_modules[]"]');
				var modules = [];
				$.each(attached_modules, function(index, module) {
					modules[index] = module.value; // get all atached modules
				});
				modules.push(module_id); // append the current module
				$.ajax({
					type: "POST",
					url: xepm_mapping.url,
					dataType: 'json',
					data: {
						class: 'endpoint',
						method: 'add_module_for_device',
						modules: modules.join(","),
						device_id: device_id
					},
					success: function (data) {
						xepm_mapping.populate_module_button_data(data.module_button_types, data.lines);
						if(data.exp_buttons != null) {
							$("<li/>", { text: module_name })
							.bind("click", function(event) { xepm_mapping.show_module(event.target); })
							.append(
								$("<i/>", { class : "fa fa-close btn" })
								.bind("click", function(event) { xepm_mapping.remove_module(event); }))
							.appendTo($("ul#mapping_attached_modules"));
							$("div#mapping_modules").append(
								$("<div>").css({'display' : 'block'}).addClass('mapping_attached_modules').append(
									$("<input>").attr({
										'type' : 'hidden',
										'name' : 'mapping_attached_modules[]',
										'value' : module_id
									}),
									xepm_mapping.add_exp_table(data.exp_buttons, xepm_mapping.attached_modules++)
								)
							);
							xepm_mapping.show_last_module();
						}
						$('button#xepm_add_module').prop('disabled', false);
					}
				});
			}
		}
	} else {
		$('button#xepm_add_module').prop('disabled', false);
	}
};

// Add expansion table to device
xepm_mapping.prototype.add_exp_table = function(exp_buttons, module_index) {
	var table = $("<table>").addClass('epm_device_conf_exp_buttons').append(
		$("<thead>").append(
			$("<tr>").append(
				$("<th>").addClass('center'),
				$("<th>").addClass('center').html('Label'),
				$("<th>").addClass('center').html('Type'),
				$("<th>").addClass('center').html('value'),
				$("<th>").addClass('center').html('Line')
			)
		),
		$("<tbody>")
	);
	var row = $("table.epm_device_conf_exp_table_hidden tbody").find('tr');
	$.each(exp_buttons, function(button_index, button) {
		var button_row = row.clone();
		button_row = xepm_mapping.add_exp_buttons(button_row, module_index, button, button_index);
		$(button_row).appendTo(table);
	});
	return table;
};

// Add expansion buttons
xepm_mapping.prototype.add_exp_buttons = function(row, module_index, button, button_index) {
	var td = $(row).find('td');
	var inputs = $(row).find('input');
	var selects = $(row).find('select');
	row.removeClass('exp_buttons_row');
	$(td[0]).text('Button ' + (parseInt(button_index) + 1));
	$(inputs[0]).val(button.label);   //label
	$(inputs[0]).attr('name', 'mapping_expansion_buttons[' + module_index + '][' + button_index + '][label]');
	$(inputs[1]).val(button.value);   //value
	$(inputs[1]).attr('name', 'mapping_expansion_buttons[' + module_index + '][' + button_index + '][value]');
	$(selects[0]).val(button.type);   //type
	$(selects[0]).attr('name', 'mapping_expansion_buttons[' + module_index + '][' + button_index + '][type]');
	$(selects[1]).val(button.line);   //line
	$(selects[1]).attr('name', 'mapping_expansion_buttons[' + module_index + '][' + button_index + '][line]');
	return row;
};

// Populate module button types and module button lines
xepm_mapping.prototype.populate_module_button_data = function(types, lines) {
	var table = $("table.epm_device_conf_exp_table_hidden tbody");
	if(types != null) {
		var selects = $("table.epm_device_conf_exp_table_hidden tbody tr").find('select');
		$(selects).find('option').empty(); // Clean the options
		selects.eq(0).fill(types, function(type) {
			this.val(type.ident);
			this.text(type.name);
		}); // populate module button types
	}
	if(lines != null) {
		selects.eq(1).fill(lines, function(line) {
			this.val(line.ident);
			this.text(line.name);
		}); // populate module button lines
	}
};

// Show the last module
xepm_mapping.prototype.show_last_module = function() {
	xepm_mapping.show_indexed_module($("ul#mapping_attached_modules li").length - 1);
};

// Show the indexed module
xepm_mapping.prototype.show_indexed_module = function(module_index) {
	if (module_index >= 0) {
		var tabs = $("ul#mapping_attached_modules li");
		var divs = $("div#mapping_modules div");
		tabs.removeClass("active");
		tabs.eq(module_index).addClass("active");
		divs.hide();
		divs.eq(module_index).show();
		xepm_mapping.visible_module = module_index;
	} else {
		delete xepm_mapping.visible_module;
	}
};

// Show settings for setting group
xepm_mapping.prototype.change_settings = function(event) {
	var group_id = $(event.target).val();
	$("div.mapping_setting_group").hide();
	$("div[id='"+group_id+"']").show();
};

// Show the clicked module
xepm_mapping.prototype.show_module = function(element) {
	xepm_mapping.show_indexed_module($(element).index());
};

// Reboot selected devices
xepm_mapping.prototype.reboot_devices = function() {
	var devices = [];
	$("input[type='checkbox']:checked").each(function() {
		devices[devices.length] = $(this).next().val();
		$(this).prop("checked", false);
	});
	if (devices.length > 0) {
		$.ajax({
			type: "POST",
			url: xepm_mapping.url,
			dataType: 'json',
			data: {
				class: 'endpoint',
				method: 'reset_devices',
				reset_devices : devices
			}
		});
	}
};

var found_elements = [];
var search_value = null;
function search_extension() {
	var found = false;
	var search = $('#epm_device_mappinggeneral div.search_field input#extension_search');
	if(search.val().length > 0) {
		var pattern = new RegExp(search.val().toLowerCase());
		var selects = $('#epm_device_mappinggeneral table.assigned_devices_table select.mapped_extensions option:selected');
		if(!(found_elements.length != 0 && search_value == search.val())) {
			found_elements = [];
			search_value = search.val();
			selects.each(function(index, value) {
				var elem = $(this)[0].parentElement;
				$(elem).removeClass('highlight');
				if(($(this).text()).length > 0) {
					if(pattern.exec($(this).text())) {
						found = true;
						found_elements.push($(elem));
						$(elem).addClass('highlight');
						$(elem).focus();
					}
				}
			});
		} else {
			found = true;
		}
		if(!found) {
			BootstrapDialog.show({
				type: 'device_mapping_search',
				title: device_mapping_title,
				message: device_mapping_no_extension,
				buttons: [{
					label: device_mapping_confirm,
					cssClass: 'btn-primary',
					action: function(dialogItself) {
						dialogItself.close();
					}
				}]
			});
		} else {
			focus_extension();
		}
	}
}

var search_idx = 0;
function focus_extension() {
	var elem = found_elements[search_idx];
	if(found_elements.length-1 > search_idx) {
		search_idx++;
	} else {
		search_idx = 0;
	}
	$(elem).focus();
	var center = $(document).height()/2;
	var dummy_elem = $('<div></div>').css({ //Use dummy element
		position: 'absolute', top: $(elem).position().top - center
	}).appendTo($(elem).parent());
	dummy_elem[0].scrollIntoView();
	dummy_elem.remove();
}

var xepm_mapping = new xepm_mapping();
xepm_mapping.assign_events();

$(function() {
	$("#epm_device_mappinggeneral").on("focus", "[name^='mapped'],[name^='unmapped_extensions']", function() {
		var element = $(this);
		var current_value = element.val();
		var device_id = $(this).closest('tr').find('input[id="mapped_device_id[]"]').val();
		if (!element.data("options_loaded")) {
			element.prop("disabled", true);
			element.addClass("loading");
			switch(element[0].getAttribute('name')) {
				case 'mapped_template_id[]': {
					method = 'template_by_device'; // template
					break;
				}
				case 'mapped_model_id[]': {
					method = 'model_by_device'; // model
					break;
				}
				case 'mapped_brand_id[]': {
					method = 'brands'; // brand
					break;
				}
				default: {
					method = 'extensions';
				}
			}
			$.ajax({
				type: "POST",
				url: xepm_mapping.url,
				dataType: 'json',
				data: {
					class: 'endpoint',
					method: method,
					device_id: device_id
				},
				success: function (data) {
					element.empty();
					element.append(new Option('', ''));
					if(data.hasOwnProperty('templates')) {
						$.each(data.templates, function(index, template) {
							element.append(new Option(template.name, template.template_id));
							if(current_value == template.template_id) {
								element[0].selectedIndex = index + 1;
							}
						});
					} else if(data.hasOwnProperty('extensions')) {
						$.each(data.extensions, function(index, extension) {
							element.append(new Option(extension.name, extension.template_id));
							if(current_value == extension.template_id) {
								element[0].selectedIndex = index + 1;
							}
						});
					} else if(data.hasOwnProperty('models')) {
						$.each(data.models, function(index, model) {
							element.append(new Option(model.name, model.model_id));
							if(current_value == model.model_id) {
								element[0].selectedIndex = index + 1;
							}
						});
					} else if(data.hasOwnProperty('brands')) {
						$.each(data.brands, function(index, brand) {
							element.append(new Option(brand.name, brand.brand_id));
							if(current_value == brand.brand_id) {
								element[0].selectedIndex = index + 1;
							}
						});
					}
				}
			}).done(function (data) {
				element.data("options_loaded", true);
				element.removeClass("loading");
				element.prop("disabled", false);
			});
		}
		return false;
	});

	$('#epm_device_mappinggeneral span.extension_search_icon').on('click', search_extension);

	$('form#epm_device_mapping_form').keydown(function(event) { //prevent form submition on "enter"
		if(event.keyCode == 13) {
			event.preventDefault();
			search_extension();
			return false;
		}
	});

	$('#epm_device_mappinggeneral button.check_uncheck').on('click', function() {
		var checkboxs = $('#epm_device_mappinggeneral input#checkbox_mapped_brand_id\\[\\]');
		if($(this).hasClass('all_checked')) {
			checkboxs.prop('checked', false);
		} else {
			checkboxs.prop('checked', true);
		}
		$(this).toggleClass('all_checked');
	});
});
